@@ -1,3 +1,4 @@
+import asyncio
 from typing import Any, Optional
 
 from transitions import EventData
@@ -22,37 +23,44 @@ class Machine:
         # e.g., "<Machine 'running'>"
         return f'<{self.__class__.__name__} {self.state!r}>'
 
-    async def after_state_change(self, _: EventData) -> None:
+    async def after_state_change(self, event: EventData) -> None:
+        if event.event and event.event.name in ('interrupt', 'terminate', 'kill'):
+            return
         await self._context.state_change(self.state)  # type: ignore
 
     async def on_enter_initialized(self, _: EventData) -> None:
         await self._context.initialize()
 
     async def on_enter_running(self, _: EventData) -> None:
-        await self._context.run()
+        self.run_finished = asyncio.Event()
+        run_started = asyncio.Event()
 
-    async def on_exit_running(self, _: EventData) -> None:
-        await self._context.finish()
-        # NOTE: Call context.finish() here rather than in on_enter_finished()
-        # because the state should be "running" until context.finish() returns.
-        # The on_enter_finished() callback is called after the state is changed.
+        async def run() -> None:
+            async with self._context.run():
+                run_started.set()
+            # await self._context.finish()
+            await self.finish()  # type: ignore
+            self.run_finished.set()
+
+        self._task = asyncio.create_task(run())
+        await run_started.wait()
 
     def send_pdb_command(self, command: str, prompt_no: int, trace_no: int) -> None:
         assert self.is_running()  # type: ignore
         self._context.send_pdb_command(command, prompt_no, trace_no)
 
-    def interrupt(self) -> None:
-        assert self.is_running()  # type: ignore
+    async def on_interrupt(self, _: EventData) -> None:
         self._context.interrupt()
 
-    def terminate(self) -> None:
-        assert self.is_running()  # type: ignore
+    async def on_terminate(self, _: EventData) -> None:
         self._context.terminate()
 
-    def kill(self) -> None:
-        assert self.is_running()  # type: ignore
+    async def on_kill(self, _: EventData) -> None:
         self._context.kill()
 
+    async def on_exit_finished(self, _: EventData) -> None:
+        await self._task
+
     def exception(self) -> Optional[BaseException]:
         assert self.is_finished()  # type: ignore
         return self._context.exception()
