@@ -5,9 +5,11 @@ import linecache
 from logging import getLogger
 from typing import Any, AsyncIterator, Optional, Tuple
 
+from apluggy import asynccontextmanager
+
 from .context import Context
 from .fsm import Machine
-from .types import PromptInfo, RunInfo, StdoutInfo, TraceInfo
+from .types import PromptInfo, PromptNotice, RunInfo, StdoutInfo, TraceInfo
 from .utils import merge_aiters
 
 
@@ -84,21 +86,28 @@ class Nextline:
     async def run(self) -> None:
         """Execute the script and wait until it exits"""
         await self._machine.run()  # type: ignore
-        await self._machine.finish()  # type: ignore
+
+    @asynccontextmanager
+    async def run_session(self) -> AsyncIterator[Nextline]:
+        await self._machine.run()  # type: ignore
+        try:
+            yield self
+        finally:
+            await self._machine.run_finished.wait()
 
     def send_pdb_command(self, command: str, prompt_no: int, trace_no: int) -> None:
         logger = getLogger(__name__)
         logger.debug(f'send_pdb_command({command!r}, {prompt_no!r}, {trace_no!r})')
         self._machine.send_pdb_command(command, prompt_no, trace_no)
 
-    def interrupt(self) -> None:
-        self._machine.interrupt()
+    async def interrupt(self) -> None:
+        await self._machine.interrupt()  # type: ignore
 
-    def terminate(self) -> None:
-        self._machine.terminate()
+    async def terminate(self) -> None:
+        await self._machine.terminate()  # type: ignore
 
-    def kill(self) -> None:
-        self._machine.kill()
+    async def kill(self) -> None:
+        await self._machine.kill()  # type: ignore
 
     def exception(self) -> Optional[BaseException]:
         """Uncaught exception from the last run"""
@@ -171,6 +180,10 @@ class Nextline:
     def subscribe_trace_info(self) -> AsyncIterator[TraceInfo]:
         return self.subscribe("trace_info")
 
+    def prompts(self) -> AsyncIterator[PromptNotice]:
+        '''Yield for each prompt. Return when the run ends.'''
+        return self.subscribe('prompt_notice')
+
     def subscribe_prompt_info(self) -> AsyncIterator[PromptInfo]:
         return self.subscribe("prompt_info")
 
